@@ -27,7 +27,7 @@
 /* values for InitRND() */
 #define NEW_RANDOMIZE			-1
 
-#define InitRND(seed)			init_random_number(0, seed)
+#define InitEngineRND(seed)		init_random_number(0, seed)
 #define InitSimpleRND(seed)		init_random_number(1, seed)
 #define RND(max)			get_random_number(0, max)
 #define SimpleRND(max)			get_random_number(1, max)
@@ -70,6 +70,8 @@
 void fprintf_line(FILE *, char *, int);
 void printf_line(char *, int);
 char *int2str(int, int);
+char *i_to_a(unsigned int);
+int log_2(unsigned int);
 
 void InitCounter(void);
 unsigned long Counter(void);
@@ -86,12 +88,15 @@ unsigned int InitRND(long);
 #endif
 
 unsigned int init_random_number(int, long);
-unsigned int get_random_number(int, unsigned int);
+unsigned int get_random_number(int, int);
 
 char *getLoginName(void);
 char *getRealName(void);
 char *getHomeDir(void);
 
+char *getBasePath(char *);
+char *getBaseName(char *);
+
 char *getPath2(char *, char *);
 char *getPath3(char *, char *, char*);
 char *getStringCat2(char *, char *);
@@ -99,7 +104,7 @@ char *getStringCopy(char *);
 char *getStringToLower(char *);
 void setString(char **, char *);
 
-void GetOptions(char **);
+void GetOptions(char **, void (*print_usage_function)(void));
 
 void SetError(char *, ...);
 char *GetError(void);
@@ -108,12 +113,13 @@ void Error(int, char *, ...);
 void *checked_malloc(unsigned long);
 void *checked_calloc(unsigned long);
 void *checked_realloc(void *, unsigned long);
+void checked_free(void *);
 
 inline void swap_numbers(int *, int *);
 inline void swap_number_pairs(int *, int *, int *, int *);
 
-short getFile16BitInteger(FILE *, int);
-void putFile16BitInteger(FILE *, short, int);
+int getFile16BitInteger(FILE *, int);
+void putFile16BitInteger(FILE *, int, int);
 int getFile32BitInteger(FILE *, int);
 void putFile32BitInteger(FILE *, int, int);
 boolean getFileChunk(FILE *, char *, int *, int);
@@ -159,10 +165,14 @@ boolean FileIsSound(char *);
 boolean FileIsMusic(char *);
 boolean FileIsArtworkType(char *, int);
 
+char *get_mapped_token(char *);
+
 int get_parameter_value(char *, char *, int);
+int get_auto_parameter_value(char *, char *);
 
 struct FileInfo *getFileListFromConfigList(struct ConfigInfo *,
-					   struct ConfigInfo *, char **, int);
+					   struct ConfigTypeInfo *,
+					   char **, int);
 void LoadArtworkConfig(struct ArtworkListInfo *);
 void ReloadCustomArtworkList(struct ArtworkListInfo *);
 void FreeCustomArtworkLists(struct ArtworkListInfo *);
