@@ -2035,7 +2035,7 @@ struct PropertyMapping *getMusicListPropertyMapping()
 }
 
 void InitSoundList(struct ConfigInfo *config_list, int num_file_list_entries,
-		   struct ConfigInfo *config_suffix_list,
+		   struct ConfigTypeInfo *config_suffix_list,
 		   char **base_prefixes, char **ext1_suffixes,
 		   char **ext2_suffixes, char **ext3_suffixes,
 		   char **ignore_tokens)
@@ -2115,7 +2115,7 @@ void InitSoundList(struct ConfigInfo *config_list, int num_file_list_entries,
 }
 
 void InitMusicList(struct ConfigInfo *config_list, int num_file_list_entries,
-		   struct ConfigInfo *config_suffix_list,
+		   struct ConfigTypeInfo *config_suffix_list,
 		   char **base_prefixes, char **ext1_suffixes,
 		   char **ext2_suffixes, char **ext3_suffixes,
 		   char **ignore_tokens)
