@@ -8,23 +8,28 @@
 *               Germany                                    *
 *               e-mail: info@artsoft.org                   *
 *----------------------------------------------------------*
-* network.c                                                *
+* netserv.c                                                *
 ***********************************************************/
 
 #include "libgame/platform.h"
 
-#if defined(PLATFORM_UNIX)
+#if defined(NETWORK_AVALIABLE)
 
 #include <fcntl.h>
 #include <sys/time.h>
 #include <signal.h>
-#include <sys/socket.h>
 #include <errno.h>
-#include <string.h>
+
+#if defined(TARGET_SDL)
+#include "main.h"
+#else
+#include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
 #include <netdb.h>
+#include <sys/select.h>			/* apparently needed for OS/2 port */
+#endif
 
 #include "libgame/libgame.h"
 
@@ -35,7 +40,12 @@ static int onceonly = 0;
 
 struct NetworkServerPlayerInfo
 {
+#if defined(TARGET_SDL)
+  TCPsocket fd;
+#else
   int fd;
+#endif
+
   char player_name[16];
   unsigned char number;
   struct NetworkServerPlayerInfo *next;
@@ -52,17 +62,23 @@ static struct NetworkServerPlayerInfo *first_player = NULL;
 
 #define NEXT(player) ((player)->next ? (player)->next : first_player)
 
+#if defined(TARGET_SDL)
+/* TODO: peer address */
+static TCPsocket lfd;		/* listening socket */
+static SDLNet_SocketSet fds;	/* socket set */
+#else
 static struct sockaddr_in saddr;
-static int lfd;
+static int lfd;			/* listening socket */
+static fd_set fds;		/* socket set */
+static int tcp = -1;
+#endif
+
 static unsigned char realbuffer[512], *buffer = realbuffer + 4;
 
 static int interrupt;
-static int tcp = -1;
 
 static unsigned long ServerFrameCounter = 0;
 
-static fd_set fds;
-
 static void addtobuffer(struct NetworkServerPlayerInfo *player,
 			unsigned char *b, int len)
 {
@@ -78,7 +94,11 @@ static void flushuser(struct NetworkServerPlayerInfo *player)
 {
   if (player->nwrite)
   {
+#if defined(TARGET_SDL)
+    SDLNet_TCP_Send(player->fd, player->writbuffer, player->nwrite);
+#else
     write(player->fd, player->writbuffer, player->nwrite);
+#endif
     player->nwrite = 0;
   }
 }
@@ -90,8 +110,9 @@ static void broadcast(struct NetworkServerPlayerInfo *except,
 
   realbuffer[0] = realbuffer[1] = realbuffer[2] = 0;
   realbuffer[3] = (unsigned char)len;
-  for (player=first_player; player; player=player->next)
-    if (player != except && (player->active || !activeonly) && player->introduced)
+  for (player = first_player; player; player = player->next)
+    if (player != except && player->introduced &&
+	(player->active || !activeonly))
       addtobuffer(player, realbuffer, 4 + len);
 }
 
@@ -114,7 +135,7 @@ static void RemovePlayer(struct NetworkServerPlayerInfo *player)
     first_player = player->next;
   else
   {
-    for (v=first_player; v; v=v->next)
+    for (v = first_player; v; v = v->next)
     {
       if (v->next && v->next == player)
       {
@@ -123,7 +144,13 @@ static void RemovePlayer(struct NetworkServerPlayerInfo *player)
       }
     }
   }
+
+#if defined(TARGET_SDL)
+  SDLNet_TCP_DelSocket(fds, player->fd);
+  SDLNet_TCP_Close(player->fd);
+#else
   close(player->fd);
+#endif
 
   if (player->introduced)
   {
@@ -146,10 +173,15 @@ static void RemovePlayer(struct NetworkServerPlayerInfo *player)
   }
 }
 
+#if defined(TARGET_SDL)
+static void AddPlayer(TCPsocket fd)
+#else
 static void AddPlayer(int fd)
+#endif
 {
   struct NetworkServerPlayerInfo *player, *v;
   unsigned char nxn;
+  boolean again = TRUE;
 
   player = checked_malloc(sizeof (struct NetworkServerPlayerInfo));
 
@@ -163,13 +195,36 @@ static void AddPlayer(int fd)
   player->action = 0;
   player->action_received = FALSE;
 
+#if defined(TARGET_SDL)
+  SDLNet_TCP_AddSocket(fds, fd);
+#endif
+
   first_player = player;
 
   nxn = 1;
 
+#if 1
+  while (again)
+  {
+    again = FALSE;
+    v = player->next;
+
+    while (v)
+    {
+      if (v->number == nxn)
+      {
+	nxn++;
+
+	again = TRUE;
+	break;
+      }
+      v = v->next;
+    }
+  }
+#else
  again:
   v = player->next;
-  while(v)
+  while (v)
   {
     if (v->number == nxn)
     {
@@ -178,11 +233,14 @@ static void AddPlayer(int fd)
     }
     v = v->next;
   }
+#endif
 
   player->number = nxn;
+#if !defined(TARGET_SDL)
   if (options.verbose)
     Error(ERR_NETWORK_SERVER, "client %d connecting from %s",
 	  nxn, inet_ntoa(saddr.sin_addr));
+#endif
   clients++;
 
   buffer[0] = 0;
@@ -234,7 +292,7 @@ static void Handle_OP_NUMBER_WANTED(struct NetworkServerPlayerInfo *player)
       Error(ERR_NETWORK_SERVER, "client %d (%s) wants to switch to # %d",
 	    player->number, player->player_name, nr_wanted);
 
-  for (v=first_player; v; v=v->next)
+  for (v = first_player; v; v = v->next)
   {
     if (v->number == nr_wanted)
     {
@@ -282,7 +340,7 @@ static void Handle_OP_PLAYER_NAME(struct NetworkServerPlayerInfo *player,
     len=16;
   memcpy(player->player_name, &buffer[2], len-2);
   player->player_name[len-2] = 0;
-  for (i=0; i<len-2; i++)
+  for (i = 0; i < len - 2; i++)
   {
     if (player->player_name[i] < ' ' || 
 	((unsigned char)(player->player_name[i]) > 0x7e &&
@@ -308,7 +366,7 @@ static void Handle_OP_PLAYER_NAME(struct NetworkServerPlayerInfo *player,
 
   if (!player->introduced)
   {
-    for (v=first_player; v; v=v->next)
+    for (v = first_player; v; v = v->next)
     {
       if (v != player && v->introduced)
       {
@@ -337,7 +395,7 @@ static void Handle_OP_START_PLAYING(struct NetworkServerPlayerInfo *player)
 	  (buffer[4] << 8) + buffer[5],
 	  &buffer[6]);
 
-  for (w=first_player; w; w=w->next)
+  for (w = first_player; w; w = w->next)
     if (w->introduced)
       w->active = 1;
 
@@ -345,7 +403,7 @@ static void Handle_OP_START_PLAYING(struct NetworkServerPlayerInfo *player)
   ServerFrameCounter = 0;
 
   /* reset player actions */
-  for (v=first_player; v; v=v->next)
+  for (v = first_player; v; v = v->next)
   {
     v->action = 0;
     v->action_received = FALSE;
@@ -378,14 +436,14 @@ static void Handle_OP_STOP_PLAYING(struct NetworkServerPlayerInfo *player)
   broadcast(NULL, 2, 0);
 }
 
-static void Handle_OP_MOVE_FIGURE(struct NetworkServerPlayerInfo *player)
+static void Handle_OP_MOVE_PLAYER(struct NetworkServerPlayerInfo *player)
 {
   struct NetworkServerPlayerInfo *v;
   int last_client_nr = 0;
   int i;
 
   /* store player action */
-  for (v=first_player; v; v=v->next)
+  for (v = first_player; v; v = v->next)
   {
     if (v->number == player->number)
     {
@@ -395,7 +453,7 @@ static void Handle_OP_MOVE_FIGURE(struct NetworkServerPlayerInfo *player)
   }
 
   /* check if server received action from each player */
-  for (v=first_player; v; v=v->next)
+  for (v = first_player; v; v = v->next)
   {
     if (!v->action_received)
       return;
@@ -405,11 +463,11 @@ static void Handle_OP_MOVE_FIGURE(struct NetworkServerPlayerInfo *player)
   }
 
   /* initialize all player actions to zero */
-  for (i=0; i<last_client_nr; i++)
+  for (i = 0; i < last_client_nr; i++)
     buffer[6 + i] = 0;
 
   /* broadcast actions of all players to all players */
-  for (v=first_player; v; v=v->next)
+  for (v = first_player; v; v = v->next)
   {
     buffer[6 + v->number-1] = v->action;
     v->action = 0;
@@ -426,18 +484,35 @@ static void Handle_OP_MOVE_FIGURE(struct NetworkServerPlayerInfo *player)
   ServerFrameCounter++;
 }
 
+#if defined(TARGET_SDL)
+/* the following is not used for a standalone server;
+   the pointer points to an integer containing the port-number */
+int NetworkServerThread(void *ptr)
+{
+  NetworkServer(*((int *) ptr), 0);
+
+  /* should never be reached */
+  return 0;
+}
+#endif
+
 void NetworkServer(int port, int serveronly)
 {
-  int i, sl, on;
+  int sl;
   struct NetworkServerPlayerInfo *player;
-  int mfd;
   int r; 
   unsigned int len;
+#if defined(TARGET_SDL)
+  IPaddress ip;
+#else
+  int i, on;
+  int is_daemon = 0;
   struct protoent *tcpproto;
   struct timeval tv;
-  int is_daemon = 0;
+  int mfd;
+#endif
 
-#ifndef NeXT
+#if defined(PLATFORM_UNIX) && !defined(PLATFORM_NEXT)
   struct sigaction sact;
 #endif
 
@@ -447,10 +522,13 @@ void NetworkServer(int port, int serveronly)
   if (!serveronly)
     onceonly = 1;
 
+#if !defined(TARGET_SDL)
   if ((tcpproto = getprotobyname("tcp")) != NULL)
     tcp = tcpproto->p_proto;
+#endif
 
-#ifdef NeXT
+#if defined(PLATFORM_UNIX)
+#if defined(PLATFORM_NEXT)
   signal(SIGPIPE, SIG_IGN);
 #else
   sact.sa_handler = SIG_IGN;
@@ -458,7 +536,28 @@ void NetworkServer(int port, int serveronly)
   sact.sa_flags = 0;
   sigaction(SIGPIPE, &sact, NULL);
 #endif
+#endif
 
+#if defined(TARGET_SDL)
+
+  /* assume that SDL is already initialized */
+#if 0
+  if (SDLNet_Init() == -1)
+    Error(ERR_EXIT_NETWORK_SERVER, "SDLNet_Init() failed");
+  atexit(SDLNet_Quit);
+#endif
+
+  if (SDLNet_ResolveHost(&ip, NULL, port) == -1)
+    Error(ERR_EXIT_NETWORK_SERVER, "SDLNet_ResolveHost() failed");
+
+  lfd = SDLNet_TCP_Open(&ip);
+  if (!lfd)
+    Error(ERR_EXIT_NETWORK_SERVER, "SDLNet_TCP_Open() failed");
+
+  fds = SDLNet_AllocSocketSet(MAX_PLAYERS+1);
+  SDLNet_TCP_AddSocket(fds, lfd);
+
+#else
 
   if ((lfd = socket(PF_INET, SOCK_STREAM, 0)) < 0)
     Error(ERR_EXIT_NETWORK_SERVER, "socket() failed");
@@ -474,12 +573,14 @@ void NetworkServer(int port, int serveronly)
     Error(ERR_EXIT_NETWORK_SERVER, "bind() failed");
 
   listen(lfd, 5);
+#endif
 
+#if !defined(TARGET_SDL)
   if (is_daemon)
   {
     /* become a daemon, breaking all ties with the controlling terminal */
     options.verbose = FALSE;
-    for (i=0; i<255; i++)
+    for (i = 0; i < 255; i++)
     {
       if (i != lfd)
 	close(i);
@@ -497,6 +598,7 @@ void NetworkServer(int port, int serveronly)
     open("/dev/null", O_WRONLY);
     open("/dev/null", O_WRONLY);
   }
+#endif
 
   if (options.verbose)
   {
@@ -505,13 +607,22 @@ void NetworkServer(int port, int serveronly)
 	  PROTOCOL_VERSION_1, PROTOCOL_VERSION_2, PROTOCOL_VERSION_3);
   }
 
-  while(1)
+  while (1)
   {
     interrupt = 0;
 
-    for (player=first_player; player; player=player->next)
+    for (player = first_player; player; player = player->next)
       flushuser(player);
 
+#if defined(TARGET_SDL)
+    if ((sl = SDLNet_CheckSockets(fds, 500000)) < 1)
+    {
+      Error(ERR_NETWORK_SERVER, SDLNet_GetError());
+      perror("SDLNet_CheckSockets");
+    }
+
+#else
+
     FD_ZERO(&fds);
     mfd = lfd;
     player = first_player;
@@ -532,6 +643,7 @@ void NetworkServer(int port, int serveronly)
       else
 	continue;
     }
+#endif
 
     if (sl < 0)
       continue;
@@ -539,6 +651,20 @@ void NetworkServer(int port, int serveronly)
     if (sl == 0)
       continue;
 
+    /* accept incoming connections */
+#if defined(TARGET_SDL)
+    if (SDLNet_SocketReady(lfd))
+    {
+      TCPsocket newsock;
+
+      newsock = SDLNet_TCP_Accept(lfd);
+
+      if (newsock)
+	AddPlayer(newsock);
+    }
+
+#else
+
     if (FD_ISSET(lfd, &fds))
     {
       int newfd, slen;
@@ -561,14 +687,27 @@ void NetworkServer(int port, int serveronly)
       }
       continue;
     }
+#endif
 
     player = first_player;
 
     do
     {
+#if defined(TARGET_SDL)
+      if (SDLNet_SocketReady(player->fd))
+#else
       if (FD_ISSET(player->fd, &fds))
+#endif
       {
-	r = read(player->fd, player->readbuffer + player->nread, MAX_BUFFER_SIZE - player->nread);
+#if defined(TARGET_SDL)
+	/* read only 1 byte, because SDLNet blocks when we want more than is
+	   in the buffer */
+	r = SDLNet_TCP_Recv(player->fd, player->readbuffer + player->nread, 1);
+#else
+	r = read(player->fd, player->readbuffer + player->nread,
+		 MAX_BUFFER_SIZE - player->nread);
+#endif
+
 	if (r <= 0)
 	{
 	  if (options.verbose)
@@ -636,8 +775,8 @@ void NetworkServer(int port, int serveronly)
 	      Handle_OP_STOP_PLAYING(player);
 	      break;
 
-	    case OP_MOVE_FIGURE:
-	      Handle_OP_MOVE_FIGURE(player);
+	    case OP_MOVE_PLAYER:
+	      Handle_OP_MOVE_PLAYER(player);
 	      break;
 
 	    case OP_BROADCAST_MESSAGE:
@@ -664,4 +803,4 @@ void NetworkServer(int port, int serveronly)
   }
 }
 
-#endif /* PLATFORM_UNIX */
+#endif /* NETWORK_AVALIABLE */
